/*
 *  ImageServlet
 *  Copyright (C) 2008 Mark Elam
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *  For more information about the GNU Public license see:
 *  http://www.gnu.org/copyleft/gpl.html
 *
 *  ImageServlet is at:
 *  http://www.subtrack.org.uk
 *
 */
package uk.org.subtrack.imaging;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 *
 * @author Mark Elam
 */
public class ReflectionImage implements ImageBean {

    protected static Log _log = LogFactory.getLog(ReflectionImage.class);
    private String text;

    public void drawGraphics(Graphics2D g2d, Integer height, Integer width) {
        Font font = new Font("Verdana", Font.BOLD, 40);
        int pad = 10;
        g2d.setFont(font);
        FontMetrics fm = g2d.getFontMetrics();
        int textWidth = fm.stringWidth(text);
        int textHeight = fm.getAscent() + fm.getDescent();
        int imageWidth = textWidth + pad;
        int imageHeight = textHeight;

        BufferedImage textImage = new BufferedImage(imageWidth, imageHeight,
                BufferedImage.TYPE_INT_ARGB);

        // Step 1: render the text.
        Graphics2D textImageG = textImage.createGraphics();
        textImageG.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                RenderingHints.VALUE_ANTIALIAS_ON);
        textImageG.setFont(font);
        textImageG.setColor(Color.CYAN);
        textImageG.drawString(text, pad / 2, fm.getAscent());
        textImageG.dispose();

        int gap = 0;
        float opacity = 0.4f;
        float fadeHeight = 0.3f;

        g2d.setPaint(new GradientPaint(0, 0, Color.black, 0, height,
                Color.darkGray));
        g2d.fillRect(0, 0, width, height);
        g2d.translate((width - imageWidth) / 2, height / 2 - imageHeight);
        g2d.drawRenderedImage(textImage, null);
        g2d.translate(0, 2 * imageHeight + gap);
        g2d.scale(1, -1);

        BufferedImage reflection = new BufferedImage(imageWidth,
                imageHeight, BufferedImage.TYPE_INT_ARGB);
        Graphics2D rg = reflection.createGraphics();
        rg.drawRenderedImage(textImage, null);
        rg.setComposite(AlphaComposite.getInstance(AlphaComposite.DST_IN));
        rg.setPaint(
                new GradientPaint(
                0, imageHeight * fadeHeight, new Color(0.0f, 0.0f, 0.0f,
                0.0f),
                0, imageHeight, new Color(0.0f, 0.0f, 0.0f, opacity)));
        rg.fillRect(0, 0, imageWidth, imageHeight);
        g2d.drawRenderedImage(reflection, null);
    }

    public void setText(String text) {
        this.text = text;
    }
}
